/*        */
#include <config.h>

#define ABS(x)               (x-_start+0x7c00)
#define STACKSEG             0x7c00
#define BOOT_PARTEND         0x01fe
#define BOOT_SIGNATURE       0xaa55

/*      (i8086) */
.code16

.text
.globl _start

/*
 *       700h,
 *  DL  .   
 *  0:700h.
 *  , 
 * CS = 0
 * IP = 7c00h
 * DL =   (   
 *      BIOS int 13h).
 *
 * :
 * :
 *  cs:ip - 0:7c00h
 *  dl    -  
 * :
 *  00000-003FF        
 *  00400-005FF       BIOS ( 1536 )
 *  00600-07BFF       ( 30208 )
 *  07C00-07DFF         ( 512 )
 *  07E00-9FFFF       ( 623104 )
 *  A0000-FFFFF       BIOS ( 393216 )
 */

/*       */
_start:
	jmp main
	nop

/*
 *     
 *   FAT12.   
 *     3.5-  (1.44 ).
 */
system_id:	.ascii "Serg    "    /*  ,                */
sector_size:	.word 512            /*                           */
sec_per_clus:	.byte 1              /* -    ( 2)           */
reserved:	.word 1              /*    ( FAT) */
fats:		.byte 2              /*   FAT                              */
dir_entries:	.word 224            /*  -   root                */
sectors:	.word 2880           /* -    fd0 < 32 Mb             */
media:		.byte 0xf0           /*                               */
fat_length:	.word 9              /*  FAT                               */
sect_track:	.word 18             /*                        */
heads:		.word 2              /*                                  */
hidden:		.long 0              /* -                             */
total_sect:	.long 0              /* -    hd > 32 Mb              */
number_direct:	.byte 0              /*                                          */
reserv:		.byte 0              /*                                     */
loader:		.byte 0x29           /*    ( 029)     */
disk_id:	.long 0              /*   ()                     */
disk_mark:	.ascii "Serg       " /*                                          */
type_fs:	.ascii "FAT12   "    /*    (ASCII)                      */

/*    */
main:
	/*     */
	cli
	xorw %ax, %ax
	movw %ax, %ds           /*            */
	movw %ax, %ss           /*             */
	movw $STACKSEG, %sp     /*          */
	sti
	
	/*      */
	movw $ABS(msg), %ax
	pushw %ax
	call puts
	addw $2, %sp
	
	/*       fd0 */
	/*    ES:BX  */
	movw $SEG_BOOT_KERNEL, %ax
	movw %ax, %es
	xorw %bx, %bx
	/*     */
	movw $0x0002, %cx
	/*     */
	movw $0x0000, %dx
	/*       */
	movw $0x0202, %ax
	int $0x13
	
	/*   ? */
	jc 1f
	
	/*   floppy */
	call kill_motor

	/* kernel   */
	movw $ABS(msg_OK), %ax
	pushw %ax
	call puts
	addw $2, %sp
	
	/*      */
	pushw %es
	call print_hex
	addw $2, %sp
	
	movw $':', %ax
	push %ax
	call putchar
	addw $2, %sp
	
	/*      */
	pushw %bx
	call print_hex
	addw $2, %ax
	
	call print_nl
	
	/*      kernel */
	ljmp $SEG_BOOT_KERNEL, $0

1:	/*     */
	movw $ABS(msg_ER), %ax
	pushw %ax
	call puts
	addw $2, %sp
1:	jmp 1b

/*
 *      ASCII   .
 * void print_hex(short int symb)
 */
print_hex:
	pushw %bp
	movw %sp, %bp
	pushw %cx
	pushw %dx

	/*   */
	movw 4(%bp), %dx
	
	/*   */
	movw $4, %cx
	
	/*       */
1:	rolw $4, %dx
	movzbw %dl, %ax
	andb $0x0f, %al
	
	/* ,     */
	cmpb $0x09, %al
	ja 2f
	/*   */
	addw $'0', %ax
	jmp 3f
2:	/*   */
	addw $('A'-0x0a), %ax

3:	pushw %ax
	call putchar
	addw $2, %sp
	loop 1b

	popw %dx
	popw %cx
	popw %bp
	ret

/*
 * int puts(char *a)
 *    '\0'   .
 */
puts:
        pushw %bp
        movw %sp, %bp
        pushw %di
	pushw %cx
	
	/*     */
	xorw %cx, %cx
	
        /*     */
        movw 4(%bp), %di
	
1:	/*       '\0' */
	movzbw (%di), %ax
	cmpb $0, %al
	je 2f
	
	/*   */
	pushw %ax
	call putchar
	addw $2, %sp
	
	/*     */
	incw %di
	incw %cx
	jmp 1b
	
2:	movw %cx, %ax
	popw %cx
	popw %di
	popw %bp
	ret

/*
 * void print_nl(void)
 *     '\n'.
 */
print_nl:
	movw $'\n', %ax
	pushw %ax
	call putchar
	addw $2, %sp
	ret

/*
 * int putchar(int a)
 *       (007)  0 
 *      .
 *   : '\n'
 *  0x10  BIOS,  00e.
 */
putchar:
        pushw %bp
        movw %sp, %bp
        pushw %bx
	
	/*   ,    '0x07' */
	movw $0x0007, %bx
	
	/*   */
	movw 4(%bp), %ax
	pushw %ax
	
	/*  00  010 */
	movb $0x0e, %ah

	/*     */
	cmpb $'\n', %al
	jne 1f

	/*    */
	int $0x10
	movb $0x0d, %al

	/*   */
1:	int $0x10
	popw %ax
	popw %bx
	popw %bp
	ret

/*
 * void kill_motor(void)
 *   floppy.
 */
kill_motor:
	pushw %dx
	movw $0x3f2, %dx
	movb $0, %al
	outb %al, %dx
	popw %dx
	ret

msg:	.asciz "Boot code starting...\n"
msg_OK:	.asciz "Loading kernel to: "
msg_ER:	.asciz "Loading kernel ERROR...\n"

. = _start + BOOT_PARTEND
.word BOOT_SIGNATURE
